"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getProblematicNodes = exports.getRefutePercentage = exports.getConsecutiveRefutes = exports.isNodeProblematic = void 0;
const NodeList = __importStar(require("./NodeList"));
const Context_1 = require("./Context");
function isNodeProblematic(node, currentCycle) {
    if (!node.refuteCycles)
        return false;
    // Check consecutive refutes
    const refuteCyclesArray = Array.from(node.refuteCycles).sort((a, b) => a - b);
    const consecutiveRefutes = getConsecutiveRefutes(refuteCyclesArray, currentCycle);
    if (consecutiveRefutes >= Context_1.config.p2p.problematicNodeConsecutiveRefuteThreshold) {
        return true;
    }
    // Check refute percentage in recent history
    const refutePercentage = getRefutePercentage(node.refuteCycles, currentCycle);
    if (refutePercentage >= Context_1.config.p2p.problematicNodeRefutePercentageThreshold) {
        return true;
    }
    return false;
}
exports.isNodeProblematic = isNodeProblematic;
function getConsecutiveRefutes(refuteCycles, currentCycle) {
    return refuteCycles[refuteCycles.length - 1] !== currentCycle ? 0 :
        refuteCycles.filter((cycle, index) => {
            return index === 0 || cycle === refuteCycles[index - 1] + 1;
        }).length;
}
exports.getConsecutiveRefutes = getConsecutiveRefutes;
function getRefutePercentage(refuteCycles, currentCycle) {
    const windowStart = Math.max(1, currentCycle - Context_1.config.p2p.problematicNodeHistoryLength + 1);
    const windowSize = Math.min(Context_1.config.p2p.problematicNodeHistoryLength, currentCycle);
    const recentRefutes = Array.from(refuteCycles)
        .filter(cycle => cycle >= windowStart && cycle <= currentCycle).length;
    return recentRefutes / windowSize;
}
exports.getRefutePercentage = getRefutePercentage;
function getProblematicNodes(prevRecord) {
    const problematicNodes = new Set();
    for (const node of NodeList.activeByIdOrder) {
        if (isNodeProblematic(node, prevRecord.counter)) {
            problematicNodes.add(node.id);
        }
    }
    // Sort by refute percentage
    return Array.from(problematicNodes).sort((a, b) => {
        const nodeA = NodeList.nodes.get(a);
        const nodeB = NodeList.nodes.get(b);
        const percentageA = getRefutePercentage(nodeA.refuteCycles, prevRecord.counter);
        const percentageB = getRefutePercentage(nodeB.refuteCycles, prevRecord.counter);
        return percentageB - percentageA;
    });
}
exports.getProblematicNodes = getProblematicNodes;
//# sourceMappingURL=ProblemNodeHandler.js.map